@@ -3,7 +3,7 @@
   -------------------
   Control class for QBrew
   -------------------
-  Copyright (c) 2006-2007 David Johnson
+  Copyright 2006-2008 David Johnson
   Please see the header file for copyright and license information.
 ***************************************************************************/
 
@@ -179,10 +179,8 @@ void QBrew::initActions()
             this, SLOT(fileSave()));
     connect(ui.actionfilesaveas, SIGNAL(triggered()),
             this, SLOT(fileSaveAs()));
-    // TODO: disable for now...
-    ui.actionfileprintpreview->setVisible(false);
-    // connect(ui.actionfileprintpreview, SIGNAL(triggered()),
-    //        this, SLOT(filePrintPreview()));
+    connect(ui.actionfileprintpreview, SIGNAL(triggered()),
+            this, SLOT(filePrintPreview()));
     connect(ui.actionfileprint, SIGNAL(triggered()),
             this, SLOT(filePrint()));
     connect(ui.actionfileexport, SIGNAL(triggered()),
@@ -210,6 +208,10 @@ void QBrew::initActions()
             this, SLOT(helpContext()));
     connect(ui.actionabout, SIGNAL(triggered()), this, SLOT(helpAbout()));
 
+#if (QT_VERSION < QT_VERSION_CHECK(4, 4, 0))
+    ui.actionfileprintpreview->setVisible(false);
+#endif
+
     // insert toolbar toggle action
     ui.maintoolbar->toggleViewAction()->setStatusTip(tr("Toggle the toolbar"));
     ui.maintoolbar->toggleViewAction()->setWhatsThis(tr("Enable or disable the Toolbar"));
@@ -461,6 +463,7 @@ void QBrew::fileExport()
         if (!QFile::exists(fname)) {
             if (fname.indexOf('.') == -1) {
                if (selected == tr(HTML_FILTER)) fname += ".html";
+               else if (selected == tr(BEERXML_FILTER)) fname += ".xml";
                else if (selected == tr(PDF_FILTER)) fname += ".pdf";
                else if (selected == tr(TEXT_FILTER)) fname += ".txt";
             }
@@ -510,7 +513,6 @@ void QBrew::fileExport()
 
 void QBrew::filePrintPreview()
 {
-    // TODO: waiting for Qt 4.4.0, and hopefully a better print infrastructure
     statusBar()->showMessage(tr("Print preview..."));
 
     recipe_->previewRecipe(textprinter_);
@@ -823,7 +825,7 @@ void QBrew::saveConfig()
 
 void QBrew::readConfig()
 {
-    QSettings config("usermode", "qbrew");
+    QSettings config;
     config.beginGroup(QString("/") + PACKAGE);
 
     // read general config
@@ -839,7 +841,7 @@ void QBrew::readConfig()
     state_.general.autobackup =
         config.value(CONF_GEN_AUTOBACKUP, state_.general.autobackup).toBool();
     state_.general.loadlast =
-        config.value(CONF_GEN_LOADLAST, state_.general.autobackup).toBool();
+        config.value(CONF_GEN_LOADLAST, state_.general.loadlast).toBool();
     state_.general.recentnum =
         config.value(CONF_GEN_RECENTNUM, state_.general.recentnum).toUInt();
     config.endGroup();
@@ -885,7 +887,7 @@ void QBrew::readConfig()
 
 void QBrew::writeConfig()
 {
-    QSettings config("usermode", "qbrew");
+    QSettings config;
     config.beginGroup(QString("/") + PACKAGE);
 
     // write general config
@@ -925,7 +927,7 @@ void QBrew::writeConfig()
 
 void QBrew::restoreState()
 {
-    QSettings config("usermode", "qbrew");
+    QSettings config;
     config.beginGroup(QString("/") + PACKAGE);
 
     // read window state
@@ -962,7 +964,7 @@ void QBrew::restoreState()
 void QBrew::saveState()
 {
     // load config from settings
-    QSettings config("usermode", "qbrew");
+    QSettings config;
     config.beginGroup(QString("/") + PACKAGE);
 
     // write window state
@@ -1001,7 +1003,7 @@ bool QBrew::openFile(const QString &filename)
             } else {
                 // load was unsuccessful
                 QMessageBox::warning(this, TITLE,
-                                     tr("Unable to load the recipe ")
+                                     tr("Unable to load the recipe %1")
                                      .arg(QFileInfo(filename_).fileName()));
                 return false;
             }
@@ -1243,6 +1245,7 @@ bool QBrew::eventFilter(QObject *obj, QEvent *event)
 
 QString QBrew::dataBase()
 {
+    // TODO: can QDesktopServices help us here?
     QString base = qApp->applicationDirPath();
 
 #if defined(Q_WS_X11)
