@@ -3,27 +3,25 @@
   -------------------
   Recipe (document) class
   -------------------
-  Copyright 2001-2007, David Johnson
+  Copyright 2001-2008, David Johnson
   Please see the header file for copyright and license information
  ***************************************************************************/
 
-#include <math.h>
+#include <cmath>
 
 #include <QApplication>
-#include <QDomDocument>
 #include <QFile>
 #include <QMessageBox>
 #include <QTextDocument>
 #include <QTextStream>
 
-#include "textprinter.h"
+#include "beerxmlreader.h"
 #include "data.h"
-#include "recipe.h"
+#include "recipereader.h"
 #include "resource.h"
+#include "textprinter.h"
 
-#ifndef HAVE_ROUND
-#define round(x) floor(x+0.5)
-#endif
+#include "recipe.h"
 
 using namespace Resource;
 
@@ -39,12 +37,50 @@ const QByteArray Recipe::ALLGRAIN_STRING = QT_TRANSLATE_NOOP("recipe", "All Grai
 // Recipe()
 // --------
 // Default constructor
+
 Recipe::Recipe(QObject *parent)
         : QObject(parent), modified_(false), title_(), brewer_(),
           size_(5.0, Volume::gallon), style_(), grains_(),  hops_(), miscs_(),
           recipenotes_(), batchnotes_(), og_(0.0), ibu_(0), srm_(0)
 { ; }
 
+//////////////////////////////////////////////////////////////////////////////
+// Recipe()
+// --------
+// Copy constructor
+
+Recipe::Recipe(const Recipe &r)
+    : QObject(0), modified_(r.modified_), title_(r.title_), brewer_(r.brewer_),
+      size_(r.size_), style_(r.style_), grains_(r.grains_), hops_(r.hops_),
+      miscs_(r.miscs_), recipenotes_(r.recipenotes_),
+      batchnotes_(r.batchnotes_), og_(r.og_), ibu_(r.ibu_), srm_(r.srm_)
+{
+}
+
+//////////////////////////////////////////////////////////////////////////////
+// operator=()
+// -----------
+// Assignment operator
+Recipe Recipe::operator=(const Recipe &r)
+{
+    if (this != &r) {
+        modified_ = r.modified_;
+        title_ = r.title_;
+        brewer_ = r.brewer_;
+        size_ = r.size_;
+        style_ = r.style_;
+        grains_ = r.grains_;
+        hops_ = r.hops_;
+        miscs_ = r.miscs_;
+        recipenotes_ = r.recipenotes_;
+        batchnotes_ = r.batchnotes_;
+        og_ = r.og_;
+        ibu_ = r.ibu_;
+        srm_ = r.srm_;
+    }
+    return *this;
+}
+
 Recipe::~Recipe()
 { ; }
 
@@ -74,14 +110,13 @@ void Recipe::newRecipe()
     srm_ = 0;
 
     setModified(false); // new documents are not in a modified state
-    emit (recipeChanged());
+    emit recipeChanged();
 }
 
 /////////////////////////////////////////////////////////////////////////////
 // nativeFormat()
 // -------------
 // Is the recipe in native format?
-// Defined as "recipe" doctype with generator or application as "qbrew"
 
 bool Recipe::nativeFormat(const QString &filename)
 {
@@ -89,38 +124,17 @@ bool Recipe::nativeFormat(const QString &filename)
     if (!datafile.open(QFile::ReadOnly | QFile::Text)) {
         // error opening file
         qWarning() << "Error: Cannot open" << filename;
-        datafile.close();
         return false;
     }
 
-    // parse document
-    QDomDocument doc;
-    bool status = doc.setContent(&datafile);
-    datafile.close();
-    if (!status) return false;
-
-    // check the document type
-    QDomElement root = doc.documentElement();
-    if (root.tagName() != tagRecipe) return false;
-
-    // check application
-    if (root.attribute(attrApplication) != PACKAGE) {
-        // check generator if no application
-        if (root.attribute(attrGenerator) != PACKAGE)
-            return false;
-    }
-
-    return true;
+    RecipeReader reader(&datafile);
+    return reader.isRecipeFormat();
 }
 
-// TODO: make BeerXML stuff a plugin?
-
 /////////////////////////////////////////////////////////////////////////////
 // beerXmlFormat()
 // ---------------
 // Is the recipe in BeerXML format?
-// Defined as XML with "<RECIPES>" and "<RECIPE>", VERSION 1
-// Note that BeerXML 1.0 is poorly designed format
 
 bool Recipe::beerXmlFormat(const QString &filename)
 {
@@ -128,38 +142,17 @@ bool Recipe::beerXmlFormat(const QString &filename)
     if (!datafile.open(QFile::ReadOnly | QFile::Text)) {
         // error opening file
         qWarning() << "Error: Cannot open" << filename;
-        datafile.close();
         return false;
     }
 
-    // parse document
-    QDomDocument doc;
-    bool status = doc.setContent(&datafile);
-    datafile.close();
-    if (!status) return false;
-
-    // check the document type
-    QDomElement root = doc.documentElement();
-    if (root.tagName() != tagRECIPES) return false;
-
-    // check for RECIPE
-    QDomElement element, sub;
-    element = root.firstChildElement(tagRECIPE);
-    if (!element.isNull()) {
-        // check VERSION
-        sub = element.firstChildElement(tagVERSION);
-        if (!sub.isNull()) {
-            if (sub.text() == "1") return true;
-        }
-    }
-
-    return false;
+    BeerXmlReader reader(&datafile);
+    return reader.isBeerXmlFormat();
 }
 
 //////////////////////////////////////////////////////////////////////////////
 // loadRecipe()
 // --------------
-// Load a recipe
+// Load a recipe. Assumes file has been checked with nativeFormat()
 
 bool Recipe::loadRecipe(const QString &filename)
 {
@@ -177,145 +170,28 @@ bool Recipe::loadRecipe(const QString &filename)
     }
 
     QApplication::setOverrideCursor(Qt::WaitCursor);
-
-    // open dom recipe
-    QDomDocument doc;
-    doc.setContent(&datafile);
-    datafile.close();
-
-    // check the doc type and stuff
-    QDomElement root = doc.documentElement();
-    if (root.tagName() != tagRecipe) {
-        // wrong file type
-        qWarning() << "Error: Wrong file type" << filename;
-        QMessageBox::warning(0, TITLE,
-                             tr("Wrong file type for %1").arg(filename));
-        QApplication::restoreOverrideCursor();
-         return false;
-    }
-
-    // check application
-    if (root.attribute(attrApplication) != PACKAGE) {
-        // check generator if no application
-        if (root.attribute(attrGenerator) != PACKAGE) {
-            qWarning() << "Not a recipe file for" << TITLE;
-            QMessageBox::warning(0, TITLE,
-                                 tr("Not a recipe for %1").arg(TITLE.data()));
-            QApplication::restoreOverrideCursor();
-             return false;
-        }
-    }
-
-    // check file version
-    if (root.attribute(attrVersion) < RECIPE_PREVIOUS) {
-        // too old of a version
-        qWarning() << "Error: Unsupported version" << filename;
-        QMessageBox::warning(0, TITLE,
-                             tr("Unsupported version %1").arg(filename));
-        QApplication::restoreOverrideCursor();
-         return false;
-    }
-
-    // Note: only use first tag if multiple single-use tags in doc
-    QDomNodeList nodes;
-    QDomElement element, sub;
-
-    // get title
-    element = root.firstChildElement(tagTitle);
-    if (element.isNull()) {
-        qDebug() << "Warning:: missing DOM element" << tagTitle;
-    } else {
-        setTitle(element.text());
-    }
-    // get brewer
-    element = root.firstChildElement(tagBrewer);
-    if (element.isNull()) {
-        qDebug() << "Warning:: missing DOM element" << tagBrewer;
-    } else {
-        setBrewer(element.text());
-    }
-    // get style
-    element = root.firstChildElement(tagStyle);
-    if (element.isNull()) {
-        qDebug() << "Warning:: missing DOM element" << tagStyle;
-    } else {
-        // TODO: load/save entire style
-        setStyle(element.text());
-    }
-    // get batch settings // TODO: eliminate this tag, use quantity, efficiency
-    element = root.firstChildElement(tagBatch);
-    if (!element.isNull()) {
-        if (element.attribute(attrQuantity) != QString()) {
-            setSize(Volume(element.attribute(attrQuantity), Volume::gallon));
-        } else if (element.attribute(attrSize) != QString()) {
-            // deprecated tag
-            setSize(Volume(element.attribute(attrSize), Volume::gallon));
-        }
-    }
-
-    // get notes
+    title_.clear();
+    brewer_.clear();
+    size_ = Data::instance()->defaultSize();
+    style_ = Data::instance()->defaultStyle();
+    grains_.clear();
+    hops_.clear();
+    miscs_.clear();
     recipenotes_.clear();
     batchnotes_.clear();
-    element = root.firstChildElement(tagNotes);
-    for (; !element.isNull(); element=element.nextSiblingElement(tagNotes)) {
-        if (element.hasAttribute(attrClass)) {
-            if (element.attribute(attrClass) == classRecipe) {
-                if (!recipenotes_.isEmpty()) recipenotes_.append('\n');
-                recipenotes_.append(element.text());
-            } else if (element.attribute(attrClass) == classBatch) {
-                if (!batchnotes_.isEmpty()) batchnotes_.append('\n');
-                batchnotes_.append(element.text());
-            }
-        }
-    }
 
-    // get all grains tags
-    grains_.clear();
-    element = root.firstChildElement(tagGrains);
-    for (; !element.isNull(); element=element.nextSiblingElement(tagGrains)) {
-        // get all grain tags
-        sub = element.firstChildElement(tagGrain);
-        for (; !sub.isNull(); sub=sub.nextSiblingElement(tagGrain)) {
-            addGrain(Grain(sub.text(),
-                           Weight(sub.attribute(attrQuantity), Weight::pound),
-                           sub.attribute(attrExtract).toDouble(),
-                           sub.attribute(attrColor).toDouble(),
-                           sub.attribute(attrType, Grain::OTHER_STRING),
-                           sub.attribute(attrUse)));
-        }
-    }
+    // parse file
+    RecipeReader reader(&datafile);
+    bool status = reader.readRecipe(this);
+    datafile.close();
 
-    // get all hops tags
-    hops_.clear();
-    element = root.firstChildElement(tagHops);
-    for (; !element.isNull(); element=element.nextSiblingElement(tagHops)) {
-            // get all hop tags
-        sub = element.firstChildElement(tagHop);
-        for (; !sub.isNull(); sub=sub.nextSiblingElement(tagHop)) {
-            // TODO: attrForm is deprecated 0.4.0
-            QString hoptype = sub.attribute(attrType);
-            if (hoptype == QString()) hoptype = sub.attribute(attrForm);
-            addHop(Hop(sub.text(),
-                       Weight(sub.attribute(attrQuantity), Weight::ounce),
-                       hoptype,
-                       sub.attribute(attrAlpha).toDouble(),
-                       sub.attribute(attrTime).toUInt()));
-        }
-    }
-
-    // get all misc tags
-    miscs_.clear();
-    element = root.firstChildElement(tagMiscs);
-    for (; !element.isNull(); element=element.nextSiblingElement(tagMiscs)) {
-        // get all hop tags
-        sub = element.firstChildElement(tagMisc);
-        for (; !sub.isNull(); sub=sub.nextSiblingElement(tagMisc)) {
-            addMisc(Misc(sub.text(),
-                         Quantity(sub.attribute(attrQuantity),
-                                  Quantity::generic),
-                         sub.attribute(attrType, Misc::OTHER_STRING),
-                         sub.attribute(attrNotes)));
-        }
+    if (!status) {
+        qWarning() << "Error: Problem reading file" << filename;
+        qWarning() << reader.errorString();
+        QMessageBox::warning(0, TITLE,
+                             tr("Error reading file %1").arg(filename));
+        QApplication::restoreOverrideCursor();
+        return false;
     }
 
     // calculate the numbers
@@ -323,7 +199,7 @@ bool Recipe::loadRecipe(const QString &filename)
 
     // just loaded recipes  are not modified
     setModified(false);
-    emit (recipeChanged());
+    emit recipeChanged();
 
     QApplication::restoreOverrideCursor();
     return true;
@@ -336,91 +212,6 @@ bool Recipe::loadRecipe(const QString &filename)
 
 bool Recipe::saveRecipe(const QString &filename)
 {
-    QApplication::setOverrideCursor(Qt::WaitCursor);
-    QDomDocument doc(tagRecipe);
-    doc.appendChild(doc.createProcessingInstruction("xml", "version=\"1.0\""));
-
-    // create the root element
-    QDomElement root = doc.createElement(doc.doctype().name());
-    root.setAttribute(attrApplication, PACKAGE);
-    root.setAttribute(attrVersion, VERSION);
-    doc.appendChild(root);
-
-    // title
-    QDomElement element = doc.createElement(tagTitle);
-    element.appendChild(doc.createTextNode(title_));
-    root.appendChild(element);
-    // brewer
-    element = doc.createElement(tagBrewer);
-    element.appendChild(doc.createTextNode(brewer_));
-    root.appendChild(element);
-    // style
-    // TODO: load/save entire style
-    element = doc.createElement(tagStyle);
-    element.appendChild(doc.createTextNode(style_.name()));
-    root.appendChild(element);
-    root.appendChild(element);
-    // batch settings
-    element = doc.createElement(tagBatch);
-    element.setAttribute(attrQuantity, size_.toString());
-    root.appendChild(element);
-    // notes
-    if (!recipenotes_.isEmpty()) {
-        element = doc.createElement(tagNotes);
-        element.setAttribute(attrClass, classRecipe);
-        element.appendChild(doc.createTextNode(recipenotes_));
-        root.appendChild(element);
-    }
-    if (!batchnotes_.isEmpty()) {
-        element = doc.createElement(tagNotes);
-        element.setAttribute(attrClass, classBatch);
-        element.appendChild(doc.createTextNode(batchnotes_));
-        root.appendChild(element);
-    }
-
-    // grains elements
-    element = doc.createElement(tagGrains);
-    QDomElement subelement;
-    foreach(Grain grain, grains_) {
-        // iterate through grain list
-        subelement = doc.createElement(tagGrain);
-        subelement.appendChild(doc.createTextNode(grain.name()));
-        subelement.setAttribute(attrQuantity, grain.weight().toString());
-        subelement.setAttribute(attrExtract, grain.extract());
-        subelement.setAttribute(attrColor, grain.color());
-        subelement.setAttribute(attrType, grain.type());
-        subelement.setAttribute(attrUse, grain.use());
-        element.appendChild(subelement);
-    }
-    root.appendChild(element);
-
-    // hops elements
-    element = doc.createElement(tagHops);
-    foreach(Hop hop, hops_) {
-        // iterate through hop list
-        subelement = doc.createElement(tagHop);
-        subelement.appendChild(doc.createTextNode(hop.name()));
-        subelement.setAttribute(attrQuantity, hop.weight().toString());
-        subelement.setAttribute(attrType, hop.type());
-        subelement.setAttribute(attrAlpha, hop.alpha());
-        subelement.setAttribute(attrTime, hop.time());
-        element.appendChild(subelement);
-    }
-    root.appendChild(element);
-
-    // miscingredients elements
-    element = doc.createElement(tagMiscs);
-    foreach(Misc misc, miscs_) {
-        // iterate through misc list
-        subelement = doc.createElement(tagMisc);
-        subelement.appendChild(doc.createTextNode(misc.name()));
-        subelement.setAttribute(attrQuantity, misc.quantity().toString());
-        subelement.setAttribute(attrType, misc.type());
-        subelement.setAttribute(attrNotes, misc.notes());
-        element.appendChild(subelement);
-    }
-    root.appendChild(element);
-
     // open file
     QFile datafile(filename);
     if (!datafile.open(QFile::WriteOnly | QFile::Text)) {
@@ -431,13 +222,14 @@ bool Recipe::saveRecipe(const QString &filename)
                              .arg(filename)
                              .arg(datafile.errorString()));
         datafile.close();
-        QApplication::restoreOverrideCursor();
         return false;
     }
 
-    // write it out
-    QTextStream data(&datafile);
-    doc.save(data, 2);
+    QApplication::setOverrideCursor(Qt::WaitCursor);
+
+    // write out xml
+    RecipeWriter writer(&datafile);
+    writer.writeRecipe(this);
     datafile.close();
 
     // recipe is saved, so set flags accordingly
@@ -453,7 +245,6 @@ bool Recipe::saveRecipe(const QString &filename)
 
 void Recipe::previewRecipe(TextPrinter *textprinter)
 {
-    // TODO: waiting for Qt 4.4.0, and hopefully a better print infrastructure
     if (!textprinter) return;
     QTextDocument document;
     document.setHtml(recipeHTML());
@@ -539,8 +330,8 @@ QString Recipe::method()
     int mash = 0;
 
     foreach(Grain grain, grains_) {
-        if (grain.use() == Grain::MASHED_STRING) mash++;
-        else if (grain.use() == Grain::EXTRACT_STRING) extract++;
+        if (grain.use().toLower() == Grain::MASHED_STRING.toLower()) mash++;
+        else if (grain.use().toLower() == Grain::EXTRACT_STRING.toLower()) extract++;
     }
 
     if (mash > 0) {
@@ -577,10 +368,10 @@ double Recipe::calcOG()
     double est = 0.0;
     foreach(Grain grain, grains_) {
         yield = grain.yield();
-        if (grain.use() == Grain::MASHED_STRING) {
+        if (grain.use().toLower() == Grain::MASHED_STRING.toLower()) {
             // adjust for mash efficiency
             yield *= Data::instance()->efficiency();
-        } else if (grain.use() == Grain::STEEPED_STRING) {
+        } else if (grain.use().toLower() == Grain::STEEPED_STRING.toLower()) {
                 // steeped grains don't yield nearly as much as mashed grains
                 yield *= Data::instance()->steepYield();
         }
@@ -628,7 +419,7 @@ int Recipe::calcRagerIBU()
     }
     // correct for boil gravity
     if (og_ > 1.050) bitterness /= 1.0 + ((og_ - 1.050) / 0.2);
-    return (int)round(bitterness);
+    return qRound(bitterness);
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -666,7 +457,7 @@ int Recipe::calcTinsethIBU()
         bitterness += ibu;
     }
     bitterness *= (GPO / LPG) / 100.0;
-    return (int)round(bitterness);
+    return qRound(bitterness);
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -698,7 +489,7 @@ int Recipe::calcSRM()
             srm += 8.4;
         }
     }
-    return (int)round(srm);
+    return qRound(srm);
 }
 
 // TODO: following formulas need to use constants
@@ -792,4 +583,3 @@ bool Recipe::modified() const
 {
     return modified_;
 }
-
